@@ -20,6 +20,8 @@ class Configuration:
         recording_enabled: bool = False,
         timeout: int = 5,
         audio_input_delay: int = 0,
+        stt_noise_reduction_type: str = "near_field",
+        stt_prewarm_model: str = "",
     ):
         self.input_type = input_type
         self.default_voice = default_voice
@@ -31,6 +33,8 @@ class Configuration:
         self.recording_enabled = recording_enabled
         self.timeout = timeout
         self.audio_input_delay = audio_input_delay
+        self.stt_noise_reduction_type = stt_noise_reduction_type
+        self.stt_prewarm_model = stt_prewarm_model
 
         if isinstance(llm_provider, str):
             self.llm_provider = LLMProvider(llm_provider)
@@ -53,4 +57,6 @@ class Configuration:
             "recording_enabled": self.recording_enabled,
             "timeout": self.timeout,
             "audio_input_delay": self.audio_input_delay,
+            "stt_noise_reduction_type": self.stt_noise_reduction_type,
+            "stt_prewarm_model": self.stt_prewarm_model,
         }
