@@ -1,10 +1,13 @@
 import json
+from datetime import datetime, timezone
 
 
 class Event:
-    def __init__(self, name: str, data: dict = {}):
+    def __init__(self, name: str, data: dict = {}, timestamp: str | None = None):
         self.name = name
         self.data = data
+        # Default to current UTC time if not provided
+        self.timestamp = timestamp or datetime.now(timezone.utc).isoformat()
 
 
 class CustomEvent(Event):
@@ -153,7 +156,12 @@ class StopRecordingEvent(Event):
 
 class STTUpdateSettingsEvent(Event):
     def __init__(
-        self, language: str = None, prompt: str = None, endpointing: int = None
+        self,
+        language: str = None,
+        prompt: str = None,
+        endpointing: int = None,
+        noise_reduction_type: str = None,
+        model: str = None,
     ):
         super().__init__(
             "stt_update_settings",
@@ -161,10 +169,29 @@ class STTUpdateSettingsEvent(Event):
                 "language": language,
                 "prompt": prompt,
                 "endpointing": endpointing,
+                "noise_reduction_type": noise_reduction_type,
+                "model": model,
             },
         )
 
 
+class STTModelSwitchedEvent(Event):
+    def __init__(
+        self,
+        from_model: str | None,
+        to_model: str | None,
+        warmup_ms: int = 0,
+        switch_delay_ms: int = 0,
+    ):
+        payload = {
+            "from_model": from_model,
+            "to_model": to_model,
+            "warmup_ms": warmup_ms,
+            "switch_delay_ms": switch_delay_ms,
+        }
+        super().__init__("stt_model_switched", payload)
+
+
 class TurnEndEvent(Event):
     def __init__(self, duration: int):
         super().__init__("turn_end", {"duration": duration})
@@ -218,6 +245,11 @@ class MetricEvent(Event):
         super().__init__(event_name, data or {})
 
 
+class SessionEndEvent(Event):
+    def __init__(self):
+        super().__init__("session_end")
+
+
 class StartSessionEvent(Event):
     def __init__(
         self,
@@ -250,12 +282,15 @@ class MergeSessionEvent(Event):
 
 
 def event_to_str(event: Event) -> str:
-    return json.dumps({"name": event.name, "data": event.data})
+    payload = {"name": event.name, "data": event.data, "timestamp": event.timestamp}
+    return json.dumps(payload)
 
 
 def event_from_str(event_str: str) -> Event:
     event = json.loads(event_str)
     name = event["name"]
+    # Default timestamp to current UTC if missing
+    timestamp = event.get("timestamp") or datetime.now(timezone.utc).isoformat()
     data = event["data"]
 
     # Dynamic handling for metrics namespace
@@ -270,6 +305,12 @@ def event_from_str(event_str: str) -> Event:
         return MetricEvent(metric_type, metric_name, payload)
 
     event_types = {
+        "stt_model_switched": lambda: STTModelSwitchedEvent(
+            data.get("from_model"),
+            data.get("to_model"),
+            data.get("warmup_ms", 0),
+            data.get("switch_delay_ms", 0),
+        ),
         "audio": lambda: AudioEvent(data.get("path")),
         "context": lambda: ContextUpdateEvent(data.get("context")),
         "error": lambda: ErrorEvent(data.get("message")),
@@ -304,7 +345,11 @@ def event_from_str(event_str: str) -> Event:
         "start_recording": lambda: StartRecordingEvent(data.get("status_callback_url")),
         "stop_recording": StopRecordingEvent,
         "stt_update_settings": lambda: STTUpdateSettingsEvent(
-            data.get("language"), data.get("prompt"), data.get("endpointing")
+            data.get("language"),
+            data.get("prompt"),
+            data.get("endpointing"),
+            data.get("noise_reduction_type"),
+            data.get("model"),
         ),
         "turn_end": lambda: TurnEndEvent(data.get("duration")),
         "turn_interrupted": TurnInterruptedEvent,
@@ -321,7 +366,7 @@ def event_from_str(event_str: str) -> Event:
             data.get("direction"),
             data.get("context"),
         ),
-        # metrics.* namespace: dynamically parse into MetricEvent
+        "session_end": SessionEndEvent,
         "testing": lambda: TestingEvent(data.get("data")),
         "start_session": lambda: StartSessionEvent(
             data.get("agent_id"),
@@ -334,7 +379,9 @@ def event_from_str(event_str: str) -> Event:
     }
 
     if name in event_types:
-        return event_types[name]()
+        e = event_types[name]()
+        e.timestamp = timestamp
+        return e
 
     raise ValueError(f"Unknown event type: {name}")
 
